package cz.upol.jj2.lab07;

import javafx.application.Application;
import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.geometry.Pos;
import javafx.scene.Scene;
import javafx.scene.control.*;
import javafx.scene.input.KeyCode;
import javafx.scene.input.KeyCodeCombination;
import javafx.scene.input.KeyCombination;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.GridPane;
import javafx.scene.layout.HBox;
import javafx.scene.paint.Color;
import javafx.scene.text.Font;
import javafx.scene.text.Text;
import javafx.stage.Modality;
import javafx.stage.Stage;

/**
 *  inspirovano seminari dr. Krajci
 *  ukazka jednoduche FX aplikace s GUI kompletne chystanym programove
 */
public class Party extends Application {

    private  Label lblPartyStatus;

    public static void main(String[] args) {
        launch(args);
    }

    private ObservableList<Person> visitors = FXCollections.observableArrayList(
            new Person("James", "Gosling"),
            new Person("Dennis", "Ritchie"),
            new Person("Guy", "Steele"),
            new Person("Simon", "Peyton Jones"),
            new Person("Ken", "Thompson"));

    @Override
    public void start(Stage primaryStage) throws Exception {
        lblPartyStatus = new Label();
        ListView<Person> listPerson = new ListView<>();
        listPerson.setItems(visitors);
        listPerson.setOnMouseClicked(e -> {
            if (e.getClickCount() > 1)
                System.out.println("Talking to " + listPerson.getSelectionModel().getSelectedItem());
        });



        // --- Menu File
        // podtrzitko pro mnemonic parsing (viz mnemonicParsing property)
        MenuItem mnuNew = new MenuItem("_Knock, knock");
        mnuNew.setAccelerator(new KeyCodeCombination(KeyCode.K, KeyCombination.CONTROL_DOWN));
        mnuNew.setOnAction(e -> newGuest(primaryStage));

        MenuItem mnuLeave = new MenuItem("_Bye, bye");
        mnuLeave.setAccelerator(new KeyCodeCombination(KeyCode.B, KeyCombination.CONTROL_DOWN));
        mnuLeave.setOnAction(e -> guestLeft(listPerson.getSelectionModel().getSelectedItem()));

        MenuItem mnuExit = new MenuItem("_End");
        mnuExit.setAccelerator(new KeyCodeCombination(KeyCode.E, KeyCombination.CONTROL_DOWN));
        mnuExit.setOnAction(e -> primaryStage.close());

        Menu menuFile = new Menu("_Party");
        menuFile.setMnemonicParsing(true);
        menuFile.getItems().addAll(mnuNew, new SeparatorMenuItem(), mnuLeave, new SeparatorMenuItem(), mnuExit);

        MenuBar menuBar = new MenuBar(menuFile);

        BorderPane root = new BorderPane();
        root.setTop(menuBar);
        root.setCenter(listPerson);
        root.setBottom(lblPartyStatus);

        visitors.addListener((ListChangeListener<? super Person>) x -> {updatePartyStatus();});
        updatePartyStatus();
        // vlastnosti okna
        primaryStage.setTitle("Party");
        primaryStage.setScene(new Scene(root, 500, 400));
        primaryStage.show();

    }

    private void newGuest(Stage primaryStage) {
        // vytvori nove modalni okno
        Stage stage = new Stage();
        stage.initOwner(primaryStage);
        stage.initModality(Modality.WINDOW_MODAL);
        stage.setTitle("Knock, knock!");

        GridPane root = new GridPane();
        root.setAlignment(Pos.CENTER);
        root.setHgap(20);
        root.setVgap(10);
        //root.setGridLinesVisible(true);

        Text lbCaption = new Text("New guest");
        lbCaption.setFont(Font.font(20));

        root.add(lbCaption, 0, 0, 2, 1); // roztazeni pres dva sloupce

        TextField txtName = new TextField();
        TextField txtSurname = new TextField();

        root.addRow(1, new Text("Name"), txtName);
        root.addRow(2, new Text("Surname"), txtSurname);

        // text, ktery se zobrazi v pripade chyby
        Text lbNotification = new Text();
        lbNotification.setFill(Color.RED);
        root.add(lbNotification, 0, 3, 2, 1);

        // tlacitka
        Button btnOk = new Button("Ok");
        btnOk.setDefaultButton(true); // stisknuti enter odpovida stisknuti tlacitka
        btnOk.setOnAction(e -> {
            String name = txtName.getText().trim();
            String surname = txtSurname.getText().trim();
            if (name.equals("") || surname.equals("")) {
                lbNotification.setText("Something is missing");
            } else {
                Person p = new Person(name, surname);
                visitors.add(p);
                System.out.println(p.toString() + " arrived!");
                stage.close();
            }
            //updatePartyStatus();
        });

        Button btnCancel = new Button("Cancel");
        btnCancel.setCancelButton(true); // stisknuti escape odpovida stisknuti tlacitka
        btnCancel.setOnAction(e -> stage.close());

        // vytvori blok tlacitek
        HBox buttons = new HBox(10);
        buttons.getChildren().addAll(btnOk, btnCancel);
        buttons.setAlignment(Pos.CENTER_RIGHT);
        root.add(buttons, 0, 4, 2, 1);

        stage.setScene(new Scene(root, 400, 300));
        stage.centerOnScreen();
        stage.show();
    }

    private void guestLeft(Person p){
        if (p != null ) {
            visitors.remove(p);
            System.out.println(p.toString() + " left!");
        }
    }

    private void updatePartyStatus(){
        lblPartyStatus.setText(switch(visitors.size()) {
            case 0 -> "Have you sent invitations?!";
            case 1 -> "Enjoying yourself?";
            case 2 -> "Game of chess?";
            case 3, 4, 5, 6, 7, 8, 9 -> "Party hard!";
            default -> "No one believes it...";
        });
    }
}
