package cz.upol.jj2.lab07;

import java.io.IOException;
import java.net.URL;
import java.util.ResourceBundle;

import javafx.collections.FXCollections;
import javafx.collections.ListChangeListener;
import javafx.collections.ObservableList;
import javafx.fxml.FXML;
import javafx.fxml.FXMLLoader;
import javafx.fxml.Initializable;
import javafx.scene.Parent;
import javafx.scene.Scene;
import javafx.scene.control.Label;
import javafx.scene.control.ListView;
import javafx.scene.input.MouseEvent;
import javafx.stage.Modality;
import javafx.stage.Stage;

public class PartyFXMLController implements Initializable {
	
	private Stage primaryStage;

	@FXML private Label lblPartyStatus;

	private ObservableList<Person> visitors = FXCollections.observableArrayList(
			new Person("James", "Gosling"),
			new Person("Dennis", "Ritchie"),
			new Person("Guy", "Steele"),
			new Person("Simon", "Peyton Jones"),
			new Person("Ken", "Thompson"));



	@FXML private ListView<Person> listGuests;
	
	@Override
	public void initialize(URL location, ResourceBundle resources) {
		listGuests.setItems(visitors);
		visitors.addListener((ListChangeListener<? super Person>) x -> {updatePartyStatus();});
		updatePartyStatus();
	}

	@FXML public void newGuest() throws IOException {
		
		FXMLLoader loader = new FXMLLoader(PartyFXMLController.class.getResource("/lab07/NewGuest.fxml"));
		Parent root = loader.load();
		
		Stage stage = new Stage();
		NewGuestController controller = loader.getController();
		controller.setStage(stage);
		controller.setGuests(visitors);
		
		stage.initOwner(this.primaryStage);
		stage.initModality(Modality.WINDOW_MODAL);
		stage.setTitle("Knock, knock!");
		stage.centerOnScreen();
		
		Scene scene = new Scene(root, 400, 250);
		scene.getStylesheets().add(PartyFXMLController.class.getResource("/lab07/NewGuest.css").toString());
		stage.setScene(scene);
		stage.show();
		
	}

	@FXML public void exitAction() {
		primaryStage.close();
	}
	
	@FXML public void listClickAction(MouseEvent e) {
		if (e.getClickCount() > 1)
			System.out.println("Talking to " + listGuests.getSelectionModel().getSelectedItem());
	}

	@FXML private void guestLeft(){
		Person p = listGuests.getSelectionModel().getSelectedItem();
		if (p != null ) {
			visitors.remove(p);
			System.out.println(p.toString() + " left!");
		}
	}

	@FXML private void updatePartyStatus(){
		lblPartyStatus.setText(switch(visitors.size()) {
			case 0 -> "Have you sent invitations?!";
			case 1 -> "Enjoying yourself?";
			case 2 -> "Game of chess?";
			case 3, 4, 5, 6, 7, 8, 9 -> "Party hard!";
			default -> "No one believes it...";
		});
	}


	public Stage getPrimaryStage() {
		return primaryStage;
	}

	public void setPrimaryStage(Stage stage) {
		this.primaryStage = stage;
	}
}
